import pandas as pd
import geopandas as gpd

cdef merge_nodes(node_dict_list):
    nodes = [pd.DataFrame(node_dict) for node_dict in node_dict_list]
    return pd.concat(nodes).reset_index(drop=True)

cpdef create_nodes_gdf(node_dict_list):
    nodes = merge_nodes(node_dict_list)
    nodes['geometry'] = gpd.points_from_xy(nodes['lon'], nodes['lat'])
    nodes = gpd.GeoDataFrame(nodes, crs='epsg:4326')
    return nodes

cpdef create_way_gdf(data_records, geometry_array):
    datasets = [v for v in data_records.values()]
    keys = list(data_records.keys())
    data = pd.DataFrame()
    for i, key in enumerate(keys):
        data[key] = datasets[i]
    data['geometry'] = geometry_array
    return gpd.GeoDataFrame(data, crs='epsg:4326')


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
import io
import re
from os.path import dirname
from os.path import join
from setuptools import find_packages
from setuptools import setup
import os
from Cython.Build import cythonize

def read(*names, **kwargs):
    with io.open(
            join(dirname(__file__), *names),
            encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()


requirements = ['geopandas',
                'pyrobuf',
                'pygeos',
                'cykhash',
                'cython',
                ]

# extensions = [Extension("pyrosm.khash.khashsets", ["pyrosm/khash/khashsets.pyx"]),
#               Extension("pyrosm.khash.khashsets", ["pyrosm/khash/khashmaps.pyx"]),
#               ]

setup(
    name='pyrosm',
    version='0.1.0',
    license='MIT',
    description='A Python tool to parse OSM data from Protobuf format into GeoDataFrame.',
    # long_description='%s\n%s' % (
    #     re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.md')),
    #     re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.md'))
    # ),
    author='Henrikki Tenkanen',
    author_email='h.tenkanen@ucl.ac.uk',
    url='https://github.com/htenkanen/pyrosm',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
    project_urls={
        #'Documentation': 'https://pyrosm.github.io/',
        'Issue Tracker': 'https://github.com/htenkanen/pyrosm/issues',
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],

    python_requires='>=3, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <3.9',
    install_requires=requirements,
    setup_requires=requirements,
    pyrobuf_modules="proto",
    ext_modules=cythonize(os.path.join("pyrosm", "*.pyx"),
                          annotate=False,
                          #compiler_directives={'linetrace': True}
                          )
)




cpdef parse_osm_data(filepath, bounding_box, exclude_relations)
cpdef get_way_data(ways, way_tags_to_keep, network_filter)
cdef get_primitive_blocks_and_string_tables(filepath)
cdef parse_dense(pblock, data, string_table, bounding_box)
cdef parse_nodes(pblock, data, bounding_box)
cdef parse_ways(data, stringtable, node_lookup)



import pytest
from pyrosm import get_path


@pytest.fixture
def test_pbf():
    pbf_path = get_path("test_pbf")
    return pbf_path


@pytest.fixture
def helsinki_pbf():
    pbf_path = get_path("helsinki_pbf")
    return pbf_path


def test_parsing_basic_elements_from_pbf(test_pbf):
    from pyrosm import OSM
    import numpy as np
    osm = OSM(filepath=test_pbf)
    osm._read_pbf()
    nodes, ways = osm._nodes, osm._way_records

    assert isinstance(nodes, list)
    assert isinstance(ways, list)

    # Required node columns
    node_cols = ['id', 'version', 'changeset', 'timestamp', 'lon', 'lat', 'tags']
    for col in node_cols:
        for node_set in nodes:
            assert col in node_set.keys()
            # Nodes should be in numpy arrays
            assert isinstance(node_set[col], np.ndarray)

            # Check shape
            assert len(node_set[col]) in [6222, 8000]

    # Check ways shape
    assert len(ways) == 2653
    for way in ways:
        assert isinstance(way, dict)

    # Required way columns
    way_cols = ['id', 'version', 'timestamp', 'nodes']
    for way in ways:
        for col in way_cols:
            assert col in way.keys()




import os
import tempfile

__all__ = ["available", "get_path"]
_module_path = os.path.dirname(__file__)
_temp_path = os.path.join(tempfile.gettempdir(), "pyrosm")

_package_files = {"test_pbf": "test.osm.pbf",
                  }
_temp_files = {"helsinki_pbf": "Helsinki.osm.pbf"}

available = list(_package_files.keys()) + list(_temp_files.keys())


def get_path(dataset):
    """
    Get the path to the data file.

    Parameters
    ----------
    dataset : str
        The name of the dataset. See ``pyrosm.data.available`` for
        all options.
    """
    if dataset in _package_files:
        return os.path.abspath(os.path.join(_module_path, _package_files[dataset]))
    elif dataset in _temp_files:
        return os.path.join(_temp_path, _temp_files[dataset])
    else:
        msg = "The dataset '{data}' is not available. ".format(data=dataset)
        msg += "Available datasets are {}".format(", ".join(available))
        raise ValueError(msg)
